<?php

/**
 * CampTix AggrePay Payment Method
 *
 * This class handles all AggrePay integration for CampTix
 *
 * @category    Class
 * @author      Stefen Rosario
 */

if (!defined('ABSPATH')) {
	exit;
} // Exit if accessed directly

class CampTix_Payment_Method_Aggrepaypayments extends CampTix_Payment_Method
{
	public $id = 'aggrepaypayments';
	public $name = 'AggrePay for CampTix';
	public $description = 'Additional plugin for Camptix to Collect Payments in Indian Rupees by various modes of transactions such as Credit/Debit Cards, Netbanking, UPI, Wallets and many more.';
	public $supported_currencies = ['INR'];

	/**
	 * We can have an array to store our options.
	 * Use $this->get_payment_options() to retrieve them.
	 */
	protected $options = [];

	function camptix_init()
	{
		$this->options = array_merge([

			'Aggrepaypayments-Api-Key'    => '',
			'Aggrepaypayments-salt'       => '',

			'sandbox' => true,
		], $this->get_payment_options());

		// IPN Listener
		//add_action( 'template_redirect', array( $this, 'template_redirect' ) );

		if ($this->is_gateway_enable()) {
			add_action('template_redirect', [$this, 'template_redirect']);
			add_action('camptix_attendee_form_additional_info', [$this, 'add_phone_field'], 10, 3);
			add_filter('camptix_form_register_complete_attendee_object', [$this, 'add_attendee_info'], 10, 3);
			add_action('camptix_checkout_update_post_meta', [$this, 'save_attendee_info'], 10, 2);
			add_filter('camptix_metabox_attendee_info_additional_rows', [$this, 'show_attendee_info'], 10, 2);
		}
		wp_register_script('camptix-aggrepaypayments-js',
			CAMPTIX_AGGREPAYPAYMENTS_URL . 'assets/js/camptix-aggrepaypayments.js', ['jquery'], false, '1.0');
		wp_enqueue_script('camptix-aggrepaypayments-js');
	}


	public function is_gateway_enable()
	{
		return isset($this->camptix_options['payment_methods'][$this->id]);
	}

	public function show_attendee_info($rows, $attendee)
	{
		if ($attendee_phone = get_post_meta($attendee->ID, 'tix_phone', true)) {
			$rows[] = [
				__('Phone Number', 'camptix-aggrepaypayments'),
				$attendee_phone,
			];
		}
		return $rows;
	}


	public function add_attendee_info($attendee, $attendee_info, $current_count)
	{
		if (!empty($_POST['tix_attendee_info'][$current_count]['phone'])) {
			$attendee->phone = trim($_POST['tix_attendee_info'][$current_count]['phone']);
		}

		return $attendee;
	}

	public function save_attendee_info($attendee_id, $attendee)
	{
		if (property_exists($attendee, 'phone')) {
			update_post_meta($attendee_id, 'tix_phone', $attendee->phone);
		}
	}

	public function add_phone_field($form_data, $current_count, $tickets_selected_count)
	{
		ob_start();
		?>
        <tr class="tix-row-phone">
            <td class="tix-required tix-left"><?php _e('Phone Number', 'camptix-aggrepaypayments'); ?>
                <span class="tix-required-star">*</span>
            </td>
			<?php $value = isset($form_data['tix_attendee_info'][$current_count]['phone']) ? $form_data['tix_attendee_info'][$current_count]['phone'] : ''; ?>
            <td class="tix-right">
                <input name="tix_attendee_info[<?php echo esc_attr($current_count); ?>][phone]" type="text" class="mobile" value="<?php echo esc_attr($value); ?>"/><br>
                <small class="message"></small>
            </td>
        </tr>
		<?php
		echo ob_get_clean();
	}


	function payment_settings_fields()
	{


		// code change by me start
		$this->add_settings_field_helper('Aggrepaypayments-Api-Key', 'AggrePay API KEY', [$this, 'field_text']);
		$this->add_settings_field_helper('Aggrepaypayments-salt', 'AggrePay SALT', [$this, 'field_text']);

		//$this->add_settings_field_helper( 'mobile-no', 'Mobile Field Name', array( $this, 'field_text' ) );

		// code change by me end

		$this->add_settings_field_helper('sandbox', __('Sandbox/Test Mode', 'camptix'), [$this, 'field_yesno'],
			__("Click on No (Radio Button) to start Collecting Live Payments right into your account.",
				'camptix')
		);
	}

	function validate_options($input)
	{
		$output = $this->options;

		if (isset($input['Aggrepaypayments-Api-Key'])) {
			$output['Aggrepaypayments-Api-Key'] = $input['Aggrepaypayments-Api-Key'];
		}
		if (isset($input['Aggrepaypayments-salt'])) {
			$output['Aggrepaypayments-salt'] = $input['Aggrepaypayments-salt'];
		}


		if (isset($input['sandbox'])) {
			$output['sandbox'] = (bool)$input['sandbox'];
		}

		return $output;
	}

	function template_redirect()
	{
		if (!isset($_REQUEST['tix_payment_method']) || 'aggrepaypayments' != $_REQUEST['tix_payment_method']) {
			return;
		}

		if (isset($_GET['tix_action'])) {
			if ('payment_cancel' == $_GET['tix_action']) {
				$this->payment_cancel();
			}

			if ('payment_return' == $_GET['tix_action']) {
				$this->payment_return();
			}

			if ('payment_notify' == $_GET['tix_action']) {
				$this->payment_notify();
			}
		}


		if ('attendee_info' == $_GET['tix_action']) {


			$merchant = $this->get_merchant_credentials();

			$data = [
				'errors' => [
					'phone' => __('Please fill in all required fields.', 'camptix-aggrepaypayments'),
				],
			];

			wp_localize_script('camptix-aggrepaypayments-js', 'camptix_isntamojo_para', $data);
		}

	}

	function payment_return()
	{
		global $camptix;

		$this->log(sprintf('Running payment_return. Request data attached.'), null, $_REQUEST);
		$this->log(sprintf('Running payment_return. Server data attached.'), null, $_SERVER);


		$payment_token = (isset($_REQUEST['tix_payment_token'])) ? trim($_REQUEST['tix_payment_token']) : '';
		$payment_id = (isset($_REQUEST['payment_id'])) ? trim($_REQUEST['payment_id']) : '';


		$payment_token = (isset($_REQUEST['tix_payment_token'])) ? trim($_REQUEST['tix_payment_token']) : '';
		if (empty($payment_token)) {
			return;
		}

		$attendees = get_posts(
			[
				'posts_per_page' => 1,
				'post_type'      => 'tix_attendee',
				'post_status'    => ['draft', 'pending', 'publish', 'cancel', 'refund', 'failed'],
				'meta_query'     => [
					[
						'key'     => 'tix_payment_token',
						'compare' => '=',
						'value'   => $payment_token,
						'type'    => 'CHAR',
					],
				],
			]
		);

		if (empty($attendees)) {
			return;
		}

		$attendee = reset($attendees);

		if ('draft' == $attendee->post_status) {
			return $this->payment_result($payment_token, CampTix_Plugin::PAYMENT_STATUS_PENDING);
		} else {
			$access_token = get_post_meta($attendee->ID, 'tix_access_token', true);
			$url = add_query_arg([
				'tix_action'       => 'access_tickets',
				'tix_access_token' => $access_token,
			], $camptix->get_tickets_url());

			wp_safe_redirect(esc_url_raw($url . '#tix'));
			die();
		}
	}

	/**
	 * Runs when PayU Money sends an ITN signal.
	 * Verify the payload and use $this->payment_result
	 * to signal a transaction result back to CampTix.
	 */
	function payment_notify()
	{
		global $camptix;

		$this->log(sprintf('Running payment_notify. Request data attached.'), null, $_REQUEST);
		$this->log(sprintf('Running payment_notify. Server data attached.'), null, $_SERVER);

		$payment_token = (isset($_REQUEST['tix_payment_token'])) ? trim($_REQUEST['tix_payment_token']) : '';

		$payload = stripslashes_deep($_REQUEST);

		/*
		Basic PHP script to handle Aggrepaypayments RAP webhook.
		*/


		$aggrepaypayments_key = $this->options['Aggrepaypayments-Api-Key'];
		$aggrepaypayments_salt = $this->options['Aggrepaypayments-salt'];
		$mobile_no = $this->options['mobile-no'];


		$data = $_POST;
		$returned_hash =  $data['hash'];
		$calculated_return_hash = $this->returnHashCalculate($aggrepaypayments_salt,$data);
		$mac_calculated = hash_hmac("sha1", implode("|", $data), $aggrepaypayments_salt);
		if ($returned_hash == $calculated_return_hash) {
			if ($data['response_code'] == 0) {
				// Payment was successful, mark it as successful in your database.
				// You can acess payment_request_id, purpose etc here.
				$this->payment_result($_REQUEST['tix_payment_token'], CampTix_Plugin::PAYMENT_STATUS_COMPLETED);
			} else {
				// Payment was unsuccessful, mark it as failed in your database.
				// You can acess payment_request_id, purpose etc here.
				$this->payment_result($_REQUEST['tix_payment_token'], CampTix_Plugin::PAYMENT_STATUS_CANCELLED);
			}
		} else {
			$this->payment_result($_REQUEST['tix_payment_token'], CampTix_Plugin::PAYMENT_STATUS_PENDING);
		}


	}

	function returnHashCalculate($salt,$input){
	/* Remove hash key if it is present */
	unset($input['hash']);
	/*Sort the array before hashing*/
	ksort($input);
	
	/*first value of hash data will be salt*/
	$hash_data = $salt;
	
	/*Create a | (pipe) separated string of all the $input values which are available in $hash_columns*/
	foreach ($input as $key=>$value) {
		if (strlen($value) > 0) {
			$hash_data .= '|' . $value;
		}
	}
	// die($hash_data);

	$hash = null;
	if (strlen($hash_data) > 0) {
		$hash = strtoupper(hash("sha512", $hash_data));
	}
		
	return $hash;
}
	public function payment_checkout($payment_token)
	{

		if (!$payment_token || empty($payment_token)) {
			return false;
		}

		if (!in_array($this->camptix_options['currency'], $this->supported_currencies)) {
			die(__('The selected currency is not supported by this payment method.', 'camptix'));
		}

		$return_url = add_query_arg([
			'tix_action'         => 'payment_return',
			'tix_payment_token'  => $payment_token,
			'tix_payment_method' => 'aggrepaypayments',
		], $this->get_tickets_url());

		$cancel_url = add_query_arg([
			'tix_action'         => 'payment_cancel',
			'tix_payment_token'  => $payment_token,
			'tix_payment_method' => 'aggrepaypayments',
		], $this->get_tickets_url());

		$notify_url = add_query_arg([
			'tix_action'         => 'payment_notify',
			'tix_payment_token'  => $payment_token,
			'tix_payment_method' => 'aggrepaypayments',
		], $this->get_tickets_url());

		$order = $this->get_order($payment_token);

		$aggrepaypayments_key = $this->options['Aggrepaypayments-Api-Key'];
		$aggrepaypayments_salt = $this->options['Aggrepaypayments-salt'];

		$order_amount = $order['total'];
		if (isset($this->camptix_options['event_name'])) {
			$productinfo = $this->camptix_options['event_name'];
		} else {
			$productinfo = 'Ticket for Order - ' . $payment_token;
		}

		$attendees = get_posts(
			[
				'post_type'   => 'tix_attendee',
				'post_status' => 'any',
				'orderby'     => 'ID',
				'order'       => 'ASC',
				'meta_query'  => [
					[
						'key'     => 'tix_payment_token',
						'compare' => '=',
						'value'   => $payment_token,
					],
				],
			]
		);

		foreach ($attendees as $attendee) {
			$tix_id = get_post(get_post_meta($attendee->ID, 'tix_ticket_id', true));
			$attendee_questions = get_post_meta($attendee->ID, 'tix_questions', true); // Array of Attendee Questons

			$email = $attendee->tix_email;
			$name = $attendee->tix_first_name . ' ' . $attendee->tix_last_name;

		}


		$info = $this->get_order($payment_token);
		$extra_info = [
			'phone' => get_post_meta($info['attendee_id'], 'tix_phone', true),

		];

		$ch = curl_init();
		$url = 'https://biz.aggrepaypayments.com/v2/getpaymentrequesturl';
		$mode = $this->options['sandbox'] ? 'TEST' : 'LIVE';

		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HEADER, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);

		$payload = [
			'api_key'                   => $aggrepaypayments_key,
			'description'               => $productinfo,
			'amount'                    => $order_amount,
			'order_id'                  => time(),
			'mode'                      => $mode,
			'currency'                  => $this->camptix_options['currency'],
			'country'                   => 'IND',
			'phone'                     => $extra_info['phone'],
			'name'                      => $name,
			'return_url'                => $notify_url,
			'email'                     => $email,
			'zip_code'                  => '123456',
			'city'                  => 'city',
		];

		$payload['hash'] = $this->hashRequestCalculate($aggrepaypayments_salt,$payload);
		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($payload));
		$response = curl_exec($ch);
		curl_close($ch);

		$json_decode = json_decode($response, true);

		$long_url = $json_decode['data']['url'];
		header('Location:' . $long_url);

		return;
	}

	function hashRequestCalculate($salt,$input)
	{
		$hash_columns = ['address_line_1', 'address_line_2', 'amount',
            'api_key', 'city', 'country', 'currency', 'description', 'email', 'mode',
            'name', 'order_id', 'phone', 'return_url', 'state', 'udf1', 'udf2', 'udf3', 'udf4', 'udf5', 'zip_code',];

		sort($hash_columns);

		$hash_data = $salt;
		foreach ($hash_columns as $column)
		{
			if (isset($input[$column]))
			{
				if (strlen($input[$column]) > 0)
				{
					$hash_data .= '|' . trim($input[$column]);
				}
			}
		}

		$hash = strtoupper(hash("sha512", $hash_data));

		return $hash;
	}
	/**
	 * Runs when the user cancels their payment during checkout at PayPal.
	 * his will simply tell CampTix to put the created attendee drafts into to Cancelled state.
	 */
	function payment_cancel()
	{
		global $camptix;

		$this->log(sprintf('Running payment_cancel. Request data attached.'), null, $_REQUEST);
		$this->log(sprintf('Running payment_cancel. Server data attached.'), null, $_SERVER);

		$payment_token = (isset($_REQUEST['tix_payment_token'])) ? trim($_REQUEST['tix_payment_token']) : '';

		if (!$payment_token) {
			die('empty token');
		}
		// Set the associated attendees to cancelled.
		return $this->payment_result($payment_token, CampTix_Plugin::PAYMENT_STATUS_CANCELLED);
	}
}

?>